import {
  scaleLinear,
  extent,
  axisLeft,
  axisBottom,
} from 'd3';

export const scatterPlot = () => {
  let width;
  let height;
  let data;
  let xValue;
  let yValue;
  let margin;
  let radius;
  let xLabel;
  let yLabel;
  //let country;

  
  const my = (selection) => {
    const x = scaleLinear()
      .domain(extent(data, xValue))
      .range([margin.left, width - margin.right]);

    const y = scaleLinear()
      .domain(extent(data, yValue))
      .range([height - margin.bottom, margin.top]);

    const marks = data.map((d) => ({
      x: x(xValue(d)),
      y: y(yValue(d)),
      country: d.country
    }));
		
    const div = d3.select("body").append("div")	
    .attr("class", "tooltip")				
    .style("opacity", 0);
    
    const circles = selection
      .selectAll('circle')
      .data(marks)
      .join('circle')
      .attr('cx', (d) => d.x)
      .attr('cy', (d) => d.y)
      .attr('r', radius)
    	.on("mouseover", (event,d) => {
        
       div.transition()
         .duration(200)
         .style("opacity", .9);
       div.html(d.country)
         .style("left", (event.pageX) + "px")
         .style("top", (event.pageY - 28) + "px");
       })
     .on("mouseout", (d) => {
       div.transition()
         .duration(500)
         .style("opacity", 0);
       });

    // TODO make it idempotent
    // We'll do this live on May 22, 2021!
    // Tune in here: https://youtu.be/U8x862hBk4s

    selection
      .selectAll('.y-axis')
      .data(marks)
      .join('g')
      .attr('class', 'y-axis')
      .attr('transform', `translate(${margin.left},0)`)
      .call(axisLeft(y))
    	.append('text')
    	.attr('class','axis-label')
    	.text(yLabel)
    	.attr('transform', 'rotate(-90)')
			.attr('x', -(margin.top + (height - margin.top - margin.bottom) / 2))
 			.attr('y', -50); // Relative to the y axis
    
    
    selection
      .selectAll('.x-axis')
      .data(marks)
      .join('g')
      .attr('class', 'x-axis')
      .attr(
        'transform',
        `translate(0,${height - margin.bottom})`
      )
      .call(axisBottom(x))
    	.append('text')
    	.attr('class','axis-label')
    	.text(xLabel)
    	.attr('x', margin.left + (width - margin.left - margin.right) / 2)
  		.attr('y', 50) // Relative to the x axis.
    	
  };

  my.width = function (_) {
    return arguments.length ? ((width = +_), my) : width;
  };

  my.height = function (_) {
    return arguments.length ? ((height = +_), my) : height;
  };

  my.data = function (_) {
    return arguments.length ? ((data = _), my) : data;
  };

  my.xValue = function (_) {
    return arguments.length ? ((xValue = _), my) : xValue;
  };

  my.yValue = function (_) {
    return arguments.length ? ((yValue = _), my) : yValue;
  };

  my.margin = function (_) {
    return arguments.length ? ((margin = _), my) : margin;
  };

  my.radius = function (_) {
    return arguments.length ? ((radius = +_), my) : radius;
  };
  
  my.xLabel = function (_) {
    return arguments.length ? ((xLabel = _), my) : xLabel;
  };
  
  my.yLabel = function (_) {
    return arguments.length ? ((yLabel = _), my) : yLabel;
  };
  
	// my.country = function (_) {
	// return arguments.length ? ((country = _), my) : country;
	// };

  return my;
};
